using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Xml.Schema;
using System.Diagnostics;

using Microsoft.BizTalk.ExplorerOM;
using Microsoft.BizTalk.Deployment;
using Microsoft.BizTalk.Component.Interop;


namespace BizTalkBill.BizTalkHelpers
{
    class SchemaRetriever
    {
        /// <summary>
        /// contains a cache of all schemas in the catalog upon first use of Schemas property
        /// </summary>
        private static SchemaCollection _schemas;
        /// <summary>
        /// caches created DocumentSpec instances
        /// </summary>
        private static Dictionary<string, DocumentSpec> _docSpecCache;

        /// <summary>
        /// initializes used static variables
        /// </summary>
        static SchemaRetriever()
        {
            _docSpecCache = new Dictionary<string, DocumentSpec>();
        }

        //[STAThread]
        //static void Main(string[] args)
        //{
        //    //			const string SCHEMANAME = "Verzekeraar.DocumentManagement.MW.VerwerkenVerzoek.VerwerkenVerzoekBE.AanvraagHypotheekOfferte";
        //    const string SCHEMANAME = "BTF2Schemas.btf2_properties_header";

        //    XmlDocument instance = GetInstanceForSchema(SCHEMANAME);
        //}

        /// <summary>
        /// provides access to all schemas within the catalog; initialized upon first use
        /// </summary>
        public static SchemaCollection Schemas
        {
            get
            {
                if (_schemas == null)
                {
                    #region retrieve the deployed schemas from the Catalog Explorer and locate the requested schema
                    // create a ConfigurationDatabase instance because that class sets the connection string
                    // we want to use
                    Int32 nRetryCount = 0;
                    ConfigurationDatabase cdb = null;
                    BtsCatalogExplorer explorer = null;

                    while ((_schemas == null) && (nRetryCount < 20))
                    {
                        try
                        {
                            cdb = new ConfigurationDatabase();

                            explorer = new BtsCatalogExplorer();

                            // assign the connection string, effectively connecting to the database
                            // TODO: optimize this if used frequently to cache the connection
                            // rpelace (local) with your production SQL server name, just as a backup
                            explorer.ConnectionString = cdb.Database.Length == 0 ? "Integrated Security=SSPI; Persist Security Info=false; Server=(local); Database=BizTalkMgmtDb;" : cdb.ConnectionString;

                            _schemas = explorer.Schemas;

                            // comment out if you don't want activity logging
                            Debugging.WriteEventLog("BizTalkHelpers", "Retrieved Schemas from BTSCatalogExplorer: Schema Count = " + _schemas.Count, EventLogEntryType.Information);

                            nRetryCount++;
                        }
                        catch (Exception ex1)
                        {
                            Debugging.WriteEventLog("BizTalkHelpers", "Error Connecting to BizTalkCatalogExplorer = " + ex1.ToString(), EventLogEntryType.Warning);
                        }
                        finally
                        {
                            if (explorer != null)
                                explorer.Dispose();
                            if (cdb != null)
                                cdb.Dispose();
                        }
                    }

                    #endregion
                }

                return _schemas;
            }
        }

        /// <summary>
        /// retrieves a requested <see cref="C:Microsoft.BizTalk.Component.Interop.DocumentSpec"/> and loads 
        /// an example instance, which is returned.
        /// </summary>
        /// <param name="schemaName">the schema to be instantiated</param>
        /// <returns>a sample instance of the retrieved schema</returns>
        public static XmlDocument GetInstanceForSchema(string schemaName)
        {
            return GetInstanceForSchema(schemaName, null);
        }

        public static XmlDocument GetInstanceForSchema(string schemaName, string rootNode)
        {
            // defines our return value. null is returned if we fail
            XmlDocument retVal = null;

            #region convert the retrieved schema into a DocumentSpec instance, create an instance and load it into our return value

            lock (_docSpecCache)
            {

                // retrieve the requested schema from the ExplorerOM model
                Schema schema = Schemas[schemaName];
                // if the local catalog contains the requested schema
                if (schema != null)
                {
                    DocumentSpec docSchema = null;

                    if (_docSpecCache.ContainsKey(schema.AssemblyQualifiedName))
                    {
                        // comment out if you don't want activity logging
                        Debugging.WriteEventLog("BizTalkHelpers", "Schema Name = " + schemaName + "[" + schema.AssemblyQualifiedName + "] Found in Cache", EventLogEntryType.Information);

                        docSchema = _docSpecCache[schema.AssemblyQualifiedName];
                    }
                    else
                    {
                        try
                        {
                            // create a DocumentSpec from it to be able to create a correct instance
                            // fix for documents with multiple root nodes - if a document does not have multiple root nodes
                            // the document cannot be referenced with '+' notation - BC
                            string sDocSpecName = schemaName;
                            if (rootNode != null)
                            {
                                sDocSpecName = string.Format("{0}+{1}", schemaName, rootNode);
                            }
                            docSchema = new DocumentSpec(sDocSpecName, schema.BtsAssembly.DisplayName);
                            _docSpecCache.Add(schema.AssemblyQualifiedName, docSchema);

                            // comment out if you don't want activity logging
                            Debugging.WriteEventLog("BizTalkHelpers", "Schema Name = " + schemaName + "[" + schema.AssemblyQualifiedName + "] Added to Cache", EventLogEntryType.Information);
                        }
                        catch (Exception ex1)
                        {
                            Debugging.WriteEventLog("BizTalkHelpers", "Error Creating DocumentSpec for Schema Name = " + schemaName + "[" + schema.AssemblyQualifiedName + "] Error = " + ex1.ToString(), EventLogEntryType.Warning);
                        }
                    }
                    StringBuilder sb = new StringBuilder();

                    // if the DocumentSpec was correctly instantiated (i.e. the schema Assembly was found)
                    if (docSchema != null)
                    {
                        using (StringWriter writer = new StringWriter(sb))
                        {
                            // instantiate our return value
                            retVal = new XmlDocument();

                            // create and instance of the schema and load it into our XmlDocument instance
                            retVal.Load(docSchema.CreateXmlInstance(writer));
                        }
                    }
                }
            }
            #endregion

            // return null if nothing was found or the instance otherwise
            return retVal;
        }
    }
}
